"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable class-methods-use-this */
const ts_types_1 = require("@injectivelabs/ts-types");
const exceptions_1 = require("@injectivelabs/exceptions");
const sdk_ts_1 = require("@injectivelabs/sdk-ts");
const index_js_1 = require("../../../utils/wallets/ninji/index.js");
const enums_js_1 = require("../../../types/enums.js");
const cosmos_js_1 = require("../../../utils/cosmos.js");
class Ninji {
    constructor(args) {
        this.chainId = args.chainId || ts_types_1.CosmosChainId.Injective;
        this.ninjiWallet = new index_js_1.NinjiWallet(args.chainId);
    }
    getWalletDeviceType() {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.resolve(enums_js_1.WalletDeviceType.Browser);
        });
    }
    enable() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield index_js_1.NinjiWallet.isChainIdSupported(this.chainId);
        });
    }
    getAddresses() {
        return __awaiter(this, void 0, void 0, function* () {
            const ninjiWallet = this.getNinjiWallet();
            try {
                const accounts = yield ninjiWallet.getAccounts();
                return accounts.map((account) => account.address);
            }
            catch (e) {
                if (e instanceof exceptions_1.CosmosWalletException) {
                    throw e;
                }
                throw new exceptions_1.CosmosWalletException(new Error(e.message), {
                    code: exceptions_1.UnspecifiedErrorCode,
                    context: enums_js_1.WalletAction.GetAccounts,
                });
            }
        });
    }
    sendTransaction(transaction, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { ninjiWallet } = this;
            const txRaw = (0, sdk_ts_1.createTxRawFromSigResponse)(transaction);
            if (!options.endpoints) {
                throw new exceptions_1.CosmosWalletException(new Error('You have to pass endpoints within the options to broadcast transaction'));
            }
            try {
                const txHash = yield ninjiWallet.broadcastTx(txRaw);
                return yield (0, sdk_ts_1.waitTxBroadcasted)(txHash, options);
            }
            catch (e) {
                throw new exceptions_1.TransactionException(new Error(e.message), {
                    code: exceptions_1.UnspecifiedErrorCode,
                    context: enums_js_1.WalletAction.SendTransaction,
                });
            }
        });
    }
    signTransaction(transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const ninjiWallet = this.getNinjiWallet();
            const signer = yield ninjiWallet.getOfflineSigner();
            const signDoc = (0, sdk_ts_1.createSignDocFromTransaction)(transaction);
            try {
                return signer.signDirect(transaction.address, (0, cosmos_js_1.createCosmosSignDocFromSignDoc)(signDoc));
            }
            catch (e) {
                if (e instanceof exceptions_1.TransactionException) {
                    throw e;
                }
                throw new exceptions_1.CosmosWalletException(new Error(e.message), {
                    code: exceptions_1.UnspecifiedErrorCode,
                    context: enums_js_1.WalletAction.SendTransaction,
                });
            }
        });
    }
    signAminoTransaction(_transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new exceptions_1.CosmosWalletException(new Error('signAminoTransaction not supported on Ninji'));
        });
    }
    getPubKey() {
        return __awaiter(this, void 0, void 0, function* () {
            const ninjiWallet = this.getNinjiWallet();
            try {
                const key = yield ninjiWallet.getKey();
                return Buffer.from(key.pubKey).toString('base64');
            }
            catch (e) {
                throw new exceptions_1.CosmosWalletException(new Error(e.message), {
                    code: exceptions_1.UnspecifiedErrorCode,
                    context: enums_js_1.WalletAction.GetAccounts,
                });
            }
        });
    }
    getNinjiWallet() {
        const { ninjiWallet } = this;
        if (!ninjiWallet) {
            throw new exceptions_1.CosmosWalletException(new Error('Please install the Ninji wallet extension'), {
                code: exceptions_1.UnspecifiedErrorCode,
                type: exceptions_1.ErrorType.WalletNotInstalledError,
                context: enums_js_1.WalletAction.SignTransaction,
            });
        }
        return ninjiWallet;
    }
}
exports.default = Ninji;
//# sourceMappingURL=Ninji.js.map