import { AccountAddress, EthereumChainId } from '@injectivelabs/ts-types';
import { DirectSignResponse } from '@cosmjs/proto-signing';
import { TxRaw, TxResponse } from '@injectivelabs/sdk-ts';
import { ConcreteWalletStrategy, EthereumWalletStrategyArgs } from '../../types/index.js';
import { SendTransactionOptions } from '../types.js';
import BaseConcreteStrategy from './Base.js';
import { WalletDeviceType } from '../../../types/enums.js';
interface PrivateKeyArgs extends EthereumWalletStrategyArgs {
    privateKey?: string;
}
export default class PrivateKey extends BaseConcreteStrategy implements ConcreteWalletStrategy {
    private privateKey?;
    constructor(args: PrivateKeyArgs);
    getWalletDeviceType(): Promise<WalletDeviceType>;
    enable(): Promise<boolean>;
    disconnect(): Promise<void>;
    getAddresses(): Promise<string[]>;
    getSessionOrConfirm(address: AccountAddress): Promise<string>;
    sendEthereumTransaction(_transaction: unknown, _options: {
        address: AccountAddress;
        ethereumChainId: EthereumChainId;
    }): Promise<string>;
    sendTransaction(transaction: TxRaw, options: SendTransactionOptions): Promise<TxResponse>;
    /** @deprecated */
    signTransaction(eip712json: string, address: AccountAddress): Promise<string>;
    signEip712TypedData(eip712json: string, address: AccountAddress): Promise<string>;
    signAminoCosmosTransaction(_transaction: {
        signDoc: any;
        accountNumber: number;
        chainId: string;
        address: string;
    }): Promise<string>;
    signCosmosTransaction(_transaction: {
        txRaw: TxRaw;
        accountNumber: number;
        chainId: string;
        address: string;
    }): Promise<DirectSignResponse>;
    signArbitrary(signer: AccountAddress, data: string | Uint8Array): Promise<string>;
    getEthereumChainId(): Promise<string>;
    getEthereumTransactionReceipt(_txHash: string): Promise<string>;
    getPubKey(): Promise<string>;
    onChainIdChanged(_callback: (chain: string) => void): Promise<void>;
    onAccountChange(_callback: (account: AccountAddress) => void): Promise<void>;
    private getPrivateKey;
}
export {};
//# sourceMappingURL=PrivateKey.d.ts.map