import { AccountAddress, EthereumChainId } from '@injectivelabs/ts-types';
import { DirectSignResponse } from '@cosmjs/proto-signing';
import { TxRaw, TxResponse } from '@injectivelabs/sdk-ts';
import { ConcreteWalletStrategy, EthereumWalletStrategyArgs } from '../../types/index.js';
import { SendTransactionOptions, WalletConnectMetadata } from '../types.js';
import BaseConcreteStrategy from './Base.js';
import { WalletDeviceType } from '../../../types/enums.js';
import Provider from '@walletconnect/ethereum-provider';
interface WalletConnectArgs extends EthereumWalletStrategyArgs {
    metadata?: WalletConnectMetadata;
}
export default class WalletConnect extends BaseConcreteStrategy implements ConcreteWalletStrategy {
    provider: Provider | undefined;
    metadata?: WalletConnectMetadata;
    constructor(args: WalletConnectArgs);
    getWalletDeviceType(): Promise<WalletDeviceType>;
    enable(args?: {
        topic: string;
    }): Promise<boolean>;
    disconnect(): Promise<void>;
    getAddresses(): Promise<string[]>;
    getSessionOrConfirm(_address: AccountAddress): Promise<string>;
    sendEthereumTransaction(transaction: unknown, _options: {
        address: AccountAddress;
        ethereumChainId: EthereumChainId;
    }): Promise<string>;
    sendTransaction(transaction: TxRaw, options: SendTransactionOptions): Promise<TxResponse>;
    /** @deprecated */
    signTransaction(eip712json: string, address: AccountAddress): Promise<string>;
    signEip712TypedData(eip712json: string, address: AccountAddress): Promise<string>;
    signAminoCosmosTransaction(_transaction: {
        signDoc: any;
        accountNumber: number;
        chainId: string;
        address: string;
    }): Promise<string>;
    signCosmosTransaction(_transaction: {
        txRaw: TxRaw;
        accountNumber: number;
        chainId: string;
        address: string;
    }): Promise<DirectSignResponse>;
    signArbitrary(signer: AccountAddress, data: string | Uint8Array): Promise<string>;
    getEthereumChainId(): Promise<string>;
    getEthereumTransactionReceipt(_txHash: string): Promise<string>;
    getPubKey(): Promise<string>;
    onChainIdChanged(callback: (chain: string) => void): Promise<void>;
    onAccountChange(callback: (account: AccountAddress) => void): Promise<void>;
    private getWalletConnect;
    private getConnectedWalletConnect;
    private connectWalletConnect;
}
export {};
//# sourceMappingURL=WalletConnect.d.ts.map