import { ethers } from 'ethers';
import { WalletException } from '@injectivelabs/exceptions';
import { getOkxWalletProvider } from '../../../strategies/wallet-strategy/strategies/Okx/utils.js';
export const getEthersProviderFromOkxWallet = async () => {
    try {
        const provider = (await getOkxWalletProvider());
        if (!provider) {
            throw new WalletException(new Error('Please install OkxWallet Extension'));
        }
        return new ethers.BrowserProvider(provider, 'any');
    }
    catch (e) {
        throw new WalletException(new Error('Please install OkxWallet Extension'));
    }
};
export const updateOkxWalletNetwork = async (chainId) => {
    try {
        const provider = (await getOkxWalletProvider());
        if (!provider) {
            throw new WalletException(new Error('Please install OkxWallet Extension'));
        }
        const chainIdToHex = chainId.toString(16);
        return await Promise.race([
            provider.request({
                method: 'wallet_switchEthereumChain',
                params: [{ chainId: `0x${chainIdToHex}` }],
            }),
            new Promise((resolve) => provider.on('change', ({ chain }) => {
                if (chain?.id === chainIdToHex) {
                    resolve();
                }
            })),
        ]);
    }
    catch (e) {
        throw new WalletException(new Error('Please update your OkxWallet network'));
    }
};
//# sourceMappingURL=Okx.js.map